@@ -25,7 +25,6 @@ from calibre.gui2 import Dispatcher, config, error_dialog, gprefs, warning_dialo
 from calibre.gui2.threaded_jobs import ThreadedJob
 from calibre.library.save_to_disk import get_components
 from calibre.utils.config import prefs, tweaks
-from calibre.utils.filenames import ascii_filename
 from calibre.utils.icu import primary_sort_key
 from calibre.utils.smtp import (
     compose_mail, config as email_config, extract_email_address, sendmail
@@ -153,11 +152,27 @@ def send_mails(jobnames, callback, attachments, to_s, subjects,
     for name, attachment, to, subject, text, aname in zip(jobnames,
             attachments, to_s, subjects, texts, attachment_names):
         description = _('Email %(name)s to %(to)s') % dict(name=name, to=to)
-        if isinstance(to, str) and ('@pbsync.com' in to or '@kindle.com' in to):
-            # The pbsync service chokes on non-ascii filenames and commas
-            # Dont know if amazon's service chokes or not, but since filenames
-            # arent visible on Kindles anyway, might as well be safe
-            aname = ascii_filename(aname).replace(',', ' ')
+        if isinstance(to, str) and ('@kindle.com' in to or '@pbsync.com' in to):
+            # The PocketBook service is a total joke. It cant handle
+            # non-ascii, filenames that are long enough to be split up, commas, and
+            # the good lord alone knows what else. So use a random filename
+            # containing only 22 English letters and numbers
+            #
+            # And since this email is only going to be processed by automated
+            # services, make the subject+text random too as at least the amazon
+            # service cant handle non-ascii text. I dont know what baboons
+            # these companies employ to write their code. It's the height of
+            # irony that they are called "tech" companies.
+            # https://bugs.launchpad.net/calibre/+bug/1989282
+            from calibre.utils.short_uuid import uuid4
+            if '@kindle.com' in to:
+                # https://www.mobileread.com/forums/showthread.php?t=349290
+                from calibre.utils.filenames import ascii_filename
+                aname = ascii_filename(aname)
+            else:
+                aname = f'{uuid4()}.' + aname.rpartition('.')[-1]
+            subject = uuid4()
+            text = uuid4()
         job = ThreadedJob('email', description, gui_sendmail, (attachment, aname, to,
                 subject, text), {}, callback)
         job_manager.run_threaded_job(job)
